package org.bitbucket.bradleysmithllc.etlunit.cli;

import org.clamshellcli.api.Command;
import org.clamshellcli.api.Configurator;
import org.clamshellcli.api.Context;
import org.clamshellcli.api.IOConsole;
import org.bitbucket.bradleysmithllc.etlunit.feature.Feature;
import org.bitbucket.bradleysmithllc.etlunit.feature.FeatureLocator;
import org.bitbucket.bradleysmithllc.etlunit.feature.ServiceLocatorFeatureLocator;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ListAvailableFeaturesCmd implements Command {
	private static final String NAMESPACE = "syscmd";
	public static final String ACTION_NAME = "list-available-features";

	public Object execute(Context ctx) {
		IOConsole console = ctx.getIoConsole();

		try {
			List<Feature> list = getAvailableFeatures();

			List<String> installed = ListInstalledFeaturesCmd.getInstalledFeatures(new File("."));

			if (list.size() != 0) {
				console.writeOutput("Available features:" + Configurator.VALUE_LINE_SEP);

				for (Feature feature : list) {
					if (installed == null || !installed.contains(feature.getFeatureName()))
					{
						console.writeOutput("\t" + feature.getFeatureName() + Configurator.VALUE_LINE_SEP);
					}
				}
			} else {
				console.writeOutput("No features available" + Configurator.VALUE_LINE_SEP);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return null;
	}

	public void plug(Context plug) {
		// no load-time setup needed
	}

	public static List<Feature> getAvailableFeatures()
	{
		ServiceLocatorFeatureLocator locator = new ServiceLocatorFeatureLocator();

		return locator.getFeatures(FeatureLocator.feature_type.external);
	}

	public Descriptor getDescriptor() {
		return new Descriptor() {
			public String getNamespace() {
				return NAMESPACE;
			}

			public String getName() {
				return ACTION_NAME;
			}

			public String getDescription() {
				return "Lists all installed features";
			}

			public String getUsage() {
				return "";
			}

			public Map<String, String> getArguments() {
				Map<String, String> map = new HashMap<String, String>();

				return map;
			}
		};
	}
}